@@ -57,6 +57,10 @@
 //#define DEBUG_3
 //#define DEBUG_4
 //#define DEBUG_4a
+//#define DEBUG_4b
+
+//#define DEBUG_4c
+//#define DEBUG_4d
 
 QTrans::QTrans(QWidget *parent, KAboutData *aboutData) : KMainWindow(parent)
 {
@@ -156,8 +160,13 @@ QTrans::QTrans(QWidget *parent, KAboutData *aboutData) : KMainWindow(parent)
 	//connect(combo,SIGNAL(returnPressed(const QString&)),   //KComboBox
 	//			this,SLOT(slot_Combo(const QString&)));
 
-	connect(combo,SIGNAL(activated(const QString&)),   //QComboBox
-				this,SLOT(slot_Combo(const QString&)));
+	//connect(combo, SIGNAL(activated(const QString&)), this, SLOT(slot_Combo(const QString&)));			//QComboBox
+	//connect(combo, SIGNAL(currentIndexChanged(const QString&)), this, SLOT(slot_Combo(const QString&)));		//ooo
+	//connect(combo, SIGNAL(editTextChanged(const QString&)), this, SLOT(slot_Combo(const QString&)));		//ooo
+
+	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(slot_Combo_2(int)));				//ooo
+	connect(combo, SIGNAL(activated(int)), this, SLOT(slot_Combo_3(int)));						//ooo
+	//connect(combo, SIGNAL(highlighted(int)), this, SLOT(slot_Combo_4(int)));					//ooo
 
 	KToolBarPopupAction* helpPopupAct = new KToolBarPopupAction(KIcon("help-browser"), i18n("&Help"), this);
 	helpPopupAct->menu()->addAction(helpAct);
@@ -222,6 +231,9 @@ QTrans::QTrans(QWidget *parent, KAboutData *aboutData) : KMainWindow(parent)
 		wordContains = true;		//ooo
 	}
 
+	//comboCurrentIndexChanged = false;	//ooo
+	comboActivatedIndexStack = 0;		//ooo
+
 	//translate if a word is copied to clipboard
 	clip = kapp->clipboard();
 	connect(clip,SIGNAL(dataChanged()),this,SLOT(slot_ClipChanged()));
@@ -337,6 +349,8 @@ void QTrans::mousePressEvent(QMouseEvent *event)	//ooo
 		//languageMenu->addSeparator();
 		languageMenu->addAction(wordStartsWithAct);	//ooo
 		languageMenu->addAction(wholeWordsOnlyAct);	//ooo
+		languageMenu->addAction(searchAllAct);		//ooo
+		languageMenu->addAction(searchEveryDictAct);	//ooo
 		languageMenu->addSeparator();
 		if (enabled) {
 			languageMenu->addAction(Dock->disableAct);
@@ -353,12 +367,17 @@ void QTrans::mousePressEvent(QMouseEvent *event)	//ooo
 
 
 /*-- Translate a QString and Update the View --*/
-void QTrans::Translate(QString EngWord, int ind)
+//void QTrans::Translate(QString EngWord, int ind)
+void QTrans::Translate(QString EngWord, int ind, bool allDict, bool inEveryDict)	//ooo
 {
 
 	QString strStatusBar;		//ooo
 	QString strTimeElapsed;		//ooo
+	QString text_check;		//ooo
+	int comboIndex = -1;		//ooo
 	NumOfLines = 0;
+	QString text = "";
+	QList<Dictionary>::const_iterator itr;   //ooo
 
 	//QMessageBox::about(this, tr("About Diagram Scene"), tr("Translate(: 1"));
 	//Out->clear();
@@ -379,24 +398,99 @@ void QTrans::Translate(QString EngWord, int ind)
 
 	Dictionary Item = bab_dict->at(ind);
 
-	if (!Dic->open(English_Path,Item.fileName)) {
-		KMessageBox::sorry(this, i18n("Could not open the dics!"));
-		return;
+	if (allDict) {
+
+		t.start();	//ooo
+
+		//for (int i = 0; i < actionsList.size(); i++) {
+		for ( itr = bab_dict->constBegin(); itr != bab_dict->constEnd(); ++itr ) {   //ooo
+
+			//Dictionary Item = bab_dict->at(i);
+
+			#ifdef DEBUG_4b
+				qDebug() << "English_Path = " << English_Path << " itr->fileName = " << itr->fileName;
+			#endif
+
+			if ( inEveryDict ) {
+				if (!Dic->open(English_Path,itr->fileName)) {
+					KMessageBox::sorry(this, i18n("Could not open the dics!"));
+					return;
+				}
+
+				text_check = Dic->translateAndFormat(EngWord, *itr, NumOfLines, wordStartsWith, wordContains, wholeWordsOnly);   //ooo
+
+				if (text_check != "") {
+
+					text += "<p>"+(*itr).fileName.section("/", -1)+"</p>";
+					//text += Dic->translateAndFormat(EngWord, *itr, NumOfLines, wordStartsWith, wordContains, wholeWordsOnly);   //ooo
+					text += text_check;
+					//text += "<hr>";
+					text += "<hr width=5 size=1 align=left>";
+					//text += "<p><spacer type=vertical size=30></p>";
+					text_check = "";
+				}
+
+				Dic->file_eng.close();
+				Dic->file_spec.close();
+			} else {
+
+				if ( (*itr).disableID ) {
+					;
+				} else {
+					if (!Dic->open(English_Path,itr->fileName)) {
+						KMessageBox::sorry(this, i18n("Could not open the dics!"));
+						return;
+					}
+
+					text_check = Dic->translateAndFormat(EngWord, *itr, NumOfLines, wordStartsWith, wordContains, wholeWordsOnly);   //ooo
+
+					if (text_check != "") {
+
+						text += "<p>"+(*itr).fileName.section("/", -1)+"</p>";
+						//text += Dic->translateAndFormat(EngWord, *itr, NumOfLines, wordStartsWith, wordContains, wholeWordsOnly);   //ooo
+						text += text_check;
+						//text += "<hr>";
+						text += "<hr width=5 size=1 align=left>";
+						//text += "<p><spacer type=vertical size=30></p>";
+						text_check = "";
+					}
+
+					Dic->file_eng.close();
+					Dic->file_spec.close();
+				}
+			}
+		}
+
+	} else {
+
+
+		//Dictionary Item = bab_dict->at(ind);
+
+		#ifdef DEBUG_4b
+			qDebug() << "English_Path = " << English_Path << " Item.fileName = " << Item.fileName;
+		#endif
+
+
+		if (!Dic->open(English_Path,Item.fileName)) {
+			KMessageBox::sorry(this, i18n("Could not open the dics!"));
+			return;
+		}
+
+		t.start();	//ooo
+
+		#ifdef DEBUG_4a
+			qDebug() << "wordStartsWith = " << wordStartsWith << ", wordContains = " << wordContains << ", wholeWordsOnly = " << wholeWordsOnly;
+		#endif
+
+		//QString text= Dic->translateAndFormat(EngWord);
+		//QString text = Dic->translateAndFormat(EngWord, Item.fileName);   //ooo
+		//QString text = Dic->translateAndFormat(EngWord, Item.fileName, Item.codecID);   //ooo
+		//QString text = Dic->translateAndFormat(EngWord, Item.fileName, Item.codecID, Item.directionID);   //ooo
+		//QString text = Dic->translateAndFormat(EngWord, Item, NumOfLines);   //ooo
+		text = Dic->translateAndFormat(EngWord, Item, NumOfLines, wordStartsWith, wordContains, wholeWordsOnly);   //ooo
+
 	}
 
-	t.start();	//ooo
-
-	#ifdef DEBUG_4a
-		qDebug() << "wordStartsWith = " << wordStartsWith << ", wordContains = " << wordContains << ", wholeWordsOnly = " << wholeWordsOnly;
-	#endif
-
-	//QString text= Dic->translateAndFormat(EngWord);
-	//QString text = Dic->translateAndFormat(EngWord, Item.fileName);   //ooo
-	//QString text = Dic->translateAndFormat(EngWord, Item.fileName, Item.codecID);   //ooo
-	//QString text = Dic->translateAndFormat(EngWord, Item.fileName, Item.codecID, Item.directionID);   //ooo
-	//QString text = Dic->translateAndFormat(EngWord, Item, NumOfLines);   //ooo
-	QString text = Dic->translateAndFormat(EngWord, Item, NumOfLines, wordStartsWith, wordContains, wholeWordsOnly);   //ooo
-
 	if (text==QString::null)
 		return;
 	if (AutoPopUp && text != "")
@@ -422,9 +516,9 @@ void QTrans::Translate(QString EngWord, int ind)
 		//QMainWindow::statusBar()->addWidget(progressBar);	//ooo
 	}
 
-#ifdef DEBUG_4
-	qDebug() << "Item.fileName: " << Item.fileName;
-#endif
+	#ifdef DEBUG_4
+		qDebug() << "Item.fileName: " << Item.fileName;
+	#endif
 
 	//QMessageBox::about(this, tr("About Diagram Scene"), tr("Translate(: 2"));
 	//Out->setOut(text);
@@ -433,10 +527,26 @@ void QTrans::Translate(QString EngWord, int ind)
 	//QMainWindow::statusBar()->showMessage("Time elapsed: %d ms", t.elapsed());	//ooo
 
 	if ( isVisible() && (EngWord != combo->currentText()) ) {
-		combo->insertItem(0, EngWord);
-		//combo->setCurrentItem(0);   //KComboBox
-		combo->setCurrentIndex(0);
+
+		comboIndex = combo->findText(EngWord, Qt::MatchCaseSensitive);	//ooo
+
+		#ifdef DEBUG_4d
+			qDebug() << "comboIndex (Translate) = " << comboIndex;
+		#endif
+
+		if (comboIndex == -1) {				//ooo
+			combo->insertItem(0, EngWord);
+			combo->setCurrentIndex(0);
+		} else {
+			//combo->removeItem(comboIndex);
+			combo->setCurrentIndex(comboIndex);
+		}
+
+		//combo->insertItem(0, EngWord);
+		   //combo->setCurrentItem(0);   //KComboBox
+		//combo->setCurrentIndex(0);
 	}
+
 	if ( combo->count() > 10 )
 		combo->removeItem(10);
 }
@@ -551,12 +661,15 @@ void QTrans::createActions()
 	stayOnTopAct = new QAction( i18n("&Stay on Top"), this );
 	stayOnTopAct->setCheckable(true);
 
-	wordStartsWithAct = new QAction( i18n("&Word begins with ..."), this );			//ooo
+	wordStartsWithAct = new QAction( i18n("&Word begins with ..."), this );				//ooo
 	wordStartsWithAct->setCheckable(true);
 
-	wholeWordsOnlyAct = new QAction( i18n("&Whole words only"), this );			//ooo
+	wholeWordsOnlyAct = new QAction( i18n("&Whole words only"), this );				//ooo
 	wholeWordsOnlyAct->setCheckable(true);
 
+	searchAllAct = new QAction( KIcon("edit-find"), i18n("&Look up in all dic"), this );		//ooo
+	searchEveryDictAct = new QAction( KIcon("system-search"), i18n("&Look up in hidden too"), this );	//ooo
+
 	disableAct = new QAction( i18n("&Disable"), this );
 	enableAct = new QAction( i18n("&Enable"), this );
 
@@ -564,8 +677,11 @@ void QTrans::createActions()
 	connect( normalModeAct, SIGNAL(triggered()), SLOT(toggleMode()) );
 	connect( stayOnTopAct, SIGNAL(triggered()), SLOT(slot_OnTop()) );
 
-	connect( wordStartsWithAct, SIGNAL(triggered()), SLOT(slot_WordStartsWith()) );		//ooo
-	connect( wholeWordsOnlyAct, SIGNAL(triggered()), SLOT(slot_WholeWordsOnly()) );		//ooo
+	connect( wordStartsWithAct, SIGNAL(triggered()), SLOT(slot_WordStartsWith()) );			//ooo
+	connect( wholeWordsOnlyAct, SIGNAL(triggered()), SLOT(slot_WholeWordsOnly()) );			//ooo
+
+	connect( searchAllAct, SIGNAL(triggered()), SLOT(slot_searchInAllDictionaries()) );		//ooo
+	connect( searchEveryDictAct, SIGNAL(triggered()), SLOT(slot_searchInEveryDictionary()) );	//ooo
 
 	connect( disableAct, SIGNAL(triggered()), SLOT(slot_Enable()) );
 	connect( enableAct, SIGNAL(triggered()), SLOT(slot_Enable()) );
@@ -707,6 +823,142 @@ void QTrans::slot_Combo(const QString &Str)
 }
 
 
+void QTrans::slot_Combo_2(int comboIndex)	//ooo
+{
+
+	#ifdef DEBUG_4c
+		qDebug() << "comboIndex               = " << comboIndex;
+		qDebug() << "combo->currentIndex()    = " << combo->currentIndex();
+		qDebug() << "combo->currentText()     = " << combo->currentText();
+		qDebug() << "comboCurrentIndexChanged = " << comboCurrentIndexChanged;
+		qDebug() << "translationExecuted      = " << translationExecuted;
+		qDebug() << "combo->count()           = " << combo->count();
+	#endif
+
+	translationEnabled = false;
+
+	switch (comboIndex) {
+		case 0:
+			if (combo->count() >=2  && !comboCurrentIndexChanged) {
+				translationExecuted = false;	//translation of first element in comboBox if combo->count() >= 2
+				break;
+			}
+
+			if (combo->count() == 1) {		//first entry in combo
+
+				#ifdef DEBUG_4c
+					qDebug() << "   * Translate *(1)* ";
+				#endif
+
+				Translate(combo->itemText(comboIndex));
+				translationExecuted = true;
+				break;
+			}
+
+			if (comboCurrentIndexChanged) {
+
+				#ifdef DEBUG_4c
+					qDebug() << "   ** Translate *(2)* ";
+				#endif
+
+				Translate(combo->itemText(comboIndex));
+				translationExecuted = true;
+				comboCurrentIndexChanged = false;
+				break;
+			}
+
+			break;
+		case 1:
+			comboCurrentIndexChanged = true;	//important by entering new words into the comboBox manually
+			translationExecuted = false;
+			break;
+		default:
+			//comboCurrentIndexChanged = true;
+			translationExecuted = false;
+	}
+
+/*	if (comboIndex > 0) {
+		//combo->insertItem(0, combo->itemText(comboIndex));
+		   //combo->addItem(combo->itemText(comboIndex));
+		//combo->removeItem(comboIndex+1);
+		//combo->setCurrentIndex(0);
+		combo->setCurrentIndex(comboIndex);
+		//Translate(combo->itemText(0));
+		Translate(combo->itemText(comboIndex));
+	}
+*/
+	#ifdef DEBUG_4c
+		//qDebug() << "comboCurrentIndexChanged = " << comboCurrentIndexChanged;
+		qDebug() << "translationExecuted      = " << translationExecuted;
+		qDebug() << "   slot_Combo_2(END)";
+	#endif
+
+}
+
+
+void QTrans::slot_Combo_3(int comboActivatedIndex)	//ooo
+{
+
+	#ifdef DEBUG_4c
+		qDebug() << "comboActivatedIndexStack = " << comboActivatedIndexStack;
+		qDebug() << "comboActivatedIndex      = " << comboActivatedIndex;
+		qDebug() << "translationExecuted      = " << translationExecuted;
+		qDebug() << "**************************";
+		//qDebug() << "combo->currentIndex() = " << combo->currentIndex();
+		//qDebug() << "combo->currentText() = " << combo->currentText();
+	#endif
+
+	if (translationEnabled && comboActivatedIndexStack == comboActivatedIndex) {
+		Translate(combo->itemText(comboActivatedIndex));
+	}
+
+	if (!translationExecuted && comboActivatedIndexStack == comboActivatedIndex) {	//first after the second passage is executed
+
+		#ifdef DEBUG_4c
+			qDebug() << "   *** Translate *(3)* ";
+		#endif
+
+		Translate(combo->itemText(comboActivatedIndex));
+		translationExecuted = true;
+		translationEnabled = true;
+	}
+
+	if (!translationExecuted /*&& comboActivatedIndex >= 0*/) {	//an entry >= 1 chosen
+		comboActivatedIndexStack = comboActivatedIndex;
+	}
+
+	if (translationExecuted) {					//(word chosen from combo + enter) + enter + enter + ..
+		translationExecuted = false;
+	}
+
+	comboCurrentIndexChanged = false;
+
+	#ifdef DEBUG_4c
+		qDebug() << "comboActivatedIndexStack = " << comboActivatedIndexStack;
+		qDebug() << "comboActivatedIndex      = " << comboActivatedIndex;
+		qDebug() << "translationExecuted      = " << translationExecuted;
+		qDebug() << "translationEnabled       = " << translationEnabled;
+		qDebug() << "slot_Combo_3(END)";
+		qDebug() << "                 ";
+	#endif
+}
+
+
+void QTrans::slot_Combo_4(int comboHighlightedIndex)	//ooo
+{
+	#ifdef DEBUG_4c
+		qDebug() << "slot_Combo_4(BEGIN)";
+		qDebug() << "comboHighlightedIndex    = " << comboHighlightedIndex;
+	#endif
+
+
+	#ifdef DEBUG_4c
+		qDebug() << "slot_Combo_4(END)";
+		qDebug() << "                 ";
+	#endif
+}
+
+
 /*-- Intercept the Close Event and decide
   if had to close or minimize --*/
 void QTrans::closeEvent(QCloseEvent *event)
@@ -895,6 +1147,8 @@ void QTrans::slot_Exit(bool ask)
 		}
 	}
 
+	//QMessageBox::about(this, tr("About Diagram Scene"), tr("slot_Exit(: 2"));
+
 	KConfigGroup cg( KGlobal::config().data(), "QTrans" );
 
 	cg.writeEntry("size", this->size());
@@ -1044,6 +1298,24 @@ void QTrans::chooseLibrary(QAction *action)
 }
 
 
+void QTrans::slot_searchInAllDictionaries()
+{
+	/*for (int i = 0; i < actionsList.size(); ++i) {
+		Translate(combo->currentText(), i);
+	}*/
+	Translate(combo->currentText(), 0, true, false);
+}
+
+
+void QTrans::slot_searchInEveryDictionary()
+{
+	/*for (int i = 0; i < actionsList.size(); ++i) {
+		Translate(combo->currentText(), i);
+	}*/
+	Translate(combo->currentText(), 0, true, true);
+}
+
+
 void QTrans::slot_About()
 {
 	KAboutApplicationDialog *dlg = new KAboutApplicationDialog(Par_aboutData, this);
@@ -1093,10 +1365,13 @@ void QTrans::slot_Ok()
 {
 	QString s = combo->currentText();
 
-	if ( combo->findText(s) )
-		combo->removeItem(0);
-	if ( combo->count() > 10 )
+	//if ( combo->findText(s) ) {
+	//	combo->removeItem(0);
+	//}
+
+	if ( combo->count() > 10 ) {
 		combo->removeItem(10);
+	}
 
 	Translate(s);
 }
