@@ -96,8 +96,13 @@ public slots:
 	void slot_WordStartsWith();		//ooo
 	void slot_WholeWordsOnly();		//ooo
 	void slot_Combo(const QString &);
+	void slot_Combo_2(int);			//ooo
+	void slot_Combo_3(int);			//ooo
+	void slot_Combo_4(int);			//ooo
 
 	void chooseLibrary(QAction *action);
+	void slot_searchInAllDictionaries();	//ooo
+	void slot_searchInEveryDictionary();	//ooo
 	void slot_aboutKDE();
 	void slot_Config();
 
@@ -107,7 +112,8 @@ public slots:
 	//void iconActivated(QSystemTrayIcon::ActivationReason reason);
 
 public:
-	void Translate(QString ,int ind=0);
+	//void Translate(QString ,int ind=0);
+	void Translate(QString ,int ind=0, bool allDict=false, bool inEveryDict=false);	//ooo
 	void readConfig();
 	void writeConfig();
 	void Default();
@@ -140,6 +146,10 @@ public:
 		StartMin,
 		MouseSelected,		//ooo
 		StatusBarSelected,	//ooo
+		comboCurrentIndexChanged,	//ooo
+		translationExecuted,	//ooo
+		enableTranslation_2,	//ooo
+		translationEnabled,	//ooo
 		enableShortCut,
 		compactMode,
 		hideTask,
@@ -159,6 +169,8 @@ public:
 	QString sentencePartTxtColor;	//ooo
 	//KComboBox *combo;
 	QComboBox *combo;
+	int comboCurrentIndexIfChanged;	//ooo
+	int comboActivatedIndexStack;	//ooo
 	QProgressBar *progressBar;	//ooo
 	QMenu *languageMenu;
 
@@ -184,6 +196,8 @@ private:
 	QAction *stayOnTopAct;
 	QAction *wordStartsWithAct;	//ooo
 	QAction *wholeWordsOnlyAct;	//ooo
+	QAction *searchAllAct;		//ooo
+	QAction *searchEveryDictAct;	//ooo
 	QAction *disableAct;
 	QAction *enableAct;
 
